function a = addressElements(A, inds)
% This function picks an element of each row of the matrix A. The target
% elements are encoded in inds.
%
% A should be a matrix and inds a vector with length(inds) = size(A,1).

%
% Copyright (c) 2015 Alexander Schulz
%
% This file is part of the Classifier Visualization Toolbox.
%
% The Classifier Visualization Toolbox is distributed under the
% GNU General Public License (version 3 or later). For details, see
% <http://www.gnu.org/licenses/>.
%


if size(A,2) == 1
    % if A is a column vector, only 1 element per row is available and
    % there is nothing to pick
    a = A;
    return
end

% Matlab saves matrices as a vector of columns, so we need to address one
% element per column.
A = A';
%[n,m] = size(A);

% if inds is not a column vector, transpose it
if (size(inds,1) == 1)
    inds = inds';
end


inds = [inds (0:length(inds)-1)'];
a = A(inds(:,1) + inds(:,2)*size(A,1));
