function [vals, svmInfo, Addr] = calcDistToNextHypPlane4SVM(model, vals, prLabels, wNorm, Addr)
% Given an SVM model, vals (= for each point x, the distance to each
% hyperplane) and prLabels (= for each point x, the classification result),
% this method picks for each point x the distance to the nearest hyperplane
% which involves the correct class.
%
% svmInfo gives the index of the 2-classSVM which was responsible for the
% smallest distance. So, abs(OrigVals(svmInfo)/wNorm(svmInfo)) = vals.

%
% Copyright (c) 2015 Alexander Schulz
%
% This file is part of the Classifier Visualization Toolbox.
%
% The Classifier Visualization Toolbox is distributed under the
% GNU General Public License (version 3 or later). For details, see
% <http://www.gnu.org/licenses/>.
%


nClasses = model.nr_class;
nSVMs    = nClasses*(nClasses-1)/2;

if (~exist('Addr', 'var'))
    % Create an adress matrix which encodes the mapping label -> indices of
    % the SVMs containing that class
    Addr = ones(nClasses, nSVMs).*Inf;
    for iClass=1:nClasses
        addr = 0;
        cl   = iClass;
        for iBlock=1:(nClasses-1)
            cl = cl - 1;
            if (cl > 0)
                % set one element in each block
                Addr(iClass, addr+cl) = 1;
            elseif (cl == 0)
                cl = -1;
                % set a whole block
                Addr(iClass, addr+1:addr+1+nClasses-(iBlock+1)) = 1;
            end
            %if (iBlock > 1)
                addr = addr + nClasses-iBlock;
            %end

        end
    end
end

% create permutation to map model.Label to usual ordering
% <>-- usual ordering is expected to be 1:nClasses --<>
usualOrderMap = zeros(nClasses,1);
for i=1:nClasses
    usualOrderMap(i) = find(model.Label == i);
end

%size(vals)
%size(Addr)
% remove all entries in val which do not contain the true class
vals = vals .* Addr(usualOrderMap(prLabels), :);
%vals = bsxfun(@rdivide, vals, wNorm(prLabels)); % wrong
vals = bsxfun(@rdivide, vals, wNorm');

[vals, svmInfo] = min(abs(vals), [], 2);



%valOld = vals';
%ind = label + (0:size(valOld,1):size(valOld,1)*(size(valOld,2)-1))';
%vals = valOld(ind); 
