function imRes = plotClassif(lowDimData, labels, samples, finalVals, ...
    finalLabels, cm, intensityScale, plotIntermResult, lowDimProts, protLabels, ...
    data2plot, labels2plot, data2plotIdx, visErrors)
% This methods receives low-dimensional data and a visualization of a
% classifier and plots these.
%
% visErrors        - if required, this matrix contains the points for which 
%                    the visualization of the classifier is incorrect

%
% Copyright (c) 2015 Alexander Schulz
%
% This file is part of the Classifier Visualization Toolbox.
%
% The Classifier Visualization Toolbox is distributed under the
% GNU General Public License (version 3 or later). For details, see
% <http://www.gnu.org/licenses/>.
%


%% pre-definitions
%posShape = [2.8 2.2 1.8];

% shift the label values, so that the smallest label is 1
labels = labels - (min(labels)-1);
nClasses = max(labels);
if ~isnan(lowDimProts(1,1))
    protLabels = protLabels - (min(labels)-1);
    nClasses = max(max(labels),max(protLabels));
end

colorVals = feval(cm, nClasses);
if nClasses==2
    colorVals = [1 0 0; 0 0 1];
    %colorVals = [0 1 0; 0 0 1]; % with green instead of red
end

symbs = {'o', 's', 'd', 'v', '^', '<', '>', 'p', 'h', '*'};
%symbs = {'o', '*', 'd', 'v', '^', '<', '>', 'p', 'h'}; % tmp for the Class vis paper
%symbs = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '0'}; %for digit data
if nClasses > length(symbs)
    % if more classes then symbols exists, repeat the latter
    symbs = repmat(symbs, 1, ceil(nClasses/length(symbs)));
end

largeSymbols = 1;
if largeSymbols
    markerS = 10; % 15 for digit data vis
    fontSizeFigs = 28;
else
    markerS = 6;
    fontSizeFigs = 24;
end

% uncomment this part in order to use the same symbol for all classes
% if numClasses > length(symbs)
%     symbs = cell(numClasses,1);
%     for i=1:numClasses
%         symbs{i} = '.';
%     end
% end



%% create contour plots: class boundaries (1) and colored class regions (2)
% plot the contours
hData = myFigure; hold on; 
colormap('gray');
pcolor(samples.samples1, samples.samples2, finalVals'); % complete contour
%contour(samples.samples1, samples.samples2, finalVals', 'k'); % additional contour lines
shading interp    % interpolation of grid values
%colorbar         % inserting colorbar
if ~isnan(intensityScale(1))
    if numel(intensityScale) == 1
        tmp = caxis;
        caxis([intensityScale tmp(2)]);
    elseif numel(intensityScale) == 2
        caxis(intensityScale);
    end
end
%alpha(0.6);     % transparency of the contour


% plot the colored class regions
hClass = myFigure;
set(hClass, 'color', 'white'); hold on;

% create the colored class regions
colormap(colorVals);
pcolor(samples.samples1, samples.samples2, finalLabels'); % complete contour
%shading interp  % interpolation of grid values
shading flat     % interpolation of grid values
%colorbar
%alpha(0.6);     % transparency of the contour

% safety measure - automatic estimation of caxis sometimes fails when only
% a single value exists
tmp = finalLabels(1,1);
if sum(finalLabels(:)==tmp) == numel(finalLabels)
    caxis([min(labels) max(labels)]);
end


% plot the data points in both plots
for fig=1:2
    if fig == 1
        figure(hData);
        descFs = round(fontSizeFigs/2.7);
    else
        figure(hClass);
        descFs = round(fontSizeFigs/2.7);
    end 

    plotHandles = zeros(nClasses,1);
    for i=1:max(nClasses)
        plotHandles(i) = plot(lowDimData(labels==i, 1), ...
            lowDimData(labels==i, 2), symbs{i}, 'MarkerSize', markerS, ...
            'MarkerFaceColor', colorVals(i,:), 'Color', colorVals(i,:));
        
        % for digit data
        %plotHandles(i) = text(lowDimData(labels==i, 1), ...
            % lowDimData(labels==i, 2), symbs{i}, 'FontSize', markerS, ...
            % 'Color', colorVals(i,:)); 

        %plot(lowDimData(LabelsSVM==i, 1), lowDimData(LabelsSVM==i, 2), '.', 'Color', colorVals(i,:)); % all data with predicted labels
        %plot(lowDimData(ldLabels==i, 1) + 10*rand(sum(ldLabels==i),1), lowDimData(ldLabels==i, 2)+ 10*rand(sum(ldLabels==i),1), '.', 'Color', colorVals(i,:)); % all data (noisy)
        %plot(ydata(trLabels==i, 1), ydata(trLabels==i, 2), '.', 'Color', colorVals(i,:)); % traindata used for tsne
        %plot(lowDimLandmarks(Landmarks_c==i, 1), lowDimLandmarks(Landmarks_c==i, 2), '.', 'Color', colorVals(i,:)); % landmarks
        %plot(ydata_ose(trLabels==i, 1), ydata_ose(trLabels==i, 2), '.', 'Color', colorVals(i,:)); % landmarks % only test data

        % plot also the additional data, if available
        if ~isempty(data2plot)
            plot(data2plot(labels2plot==i, 1),  ...
                data2plot(labels2plot==i, 2), symbs{i}, 'MarkerSize', markerS, ...
                'MarkerFaceColor', colorVals(i,:), 'Color', colorVals(i,:));
            
            uniqueSets = unique(data2plotIdx);
            if numel(uniqueSets) > numel(symbs)
                warning(['The Number of sets of additional data is larger' ...
                ' then the numer of symbols. Marking only the first ' ...
                '%d sets. Please add symbols to the variable >symbs<'], ...
                numel(symbs));
            end
            
            for numSet = 1:numel(uniqueSets)
                data2plotTmp   = data2plot(data2plotIdx==numSet,:);
                labels2plotTmp = labels2plot(data2plotIdx==numSet);
                
                plot(data2plotTmp(labels2plotTmp==i, 1),  ...
                    data2plotTmp(labels2plotTmp==i, 2), [symbs{numSet} 'k'], ...
                    'MarkerSize', round(markerS/2));
            end
        end
    end
    
    % plot the prototypes if available
    plotHandlesProtos = zeros(nClasses,1);
    if ~isnan(lowDimProts(1,1))
        protSize = 22; % old was 18
        % shift the labels
        shift  = (min(protLabels)-1);
        protLabels = protLabels - shift;
        for i=1:max(nClasses)
            plot(lowDimProts(protLabels==i,1), lowDimProts(protLabels==i,2), 'o', 'MarkerSize', protSize, 'MarkerFaceColor', 'w', 'Color', colorVals(i,:));
            plotHandlesProtos(i) = plot(lowDimProts(protLabels==i,1), lowDimProts(protLabels==i,2), '*', 'MarkerSize', protSize, 'Color', colorVals(i,:));
            %plot(lowDimProts(protLabels==i,1), lowDimProts(protLabels==i,2), 'o','markeredgecolor','k','markerfacecolor',colorVals(i,:),'markersize',9);
        end
    end

    hAxTest = get(gcf, 'CurrentAxes');
    set(hAxTest, 'FontSize', fontSizeFigs);

    range = zeros(1,4);
    range([1 3]) = min(lowDimData);
    range([2 4]) = max(lowDimData);
    range = range * 1.05;


    %range = [-1.1 1 -1.1 1.0];
    axis(range);
    set(gca,'XTickLabel','','yTickLabel','','xtick',-100:10:-110,'ytick',-100:10:-110);
    box on;
    
    % plot data points for which the classifier visualization is incorrect
    if ~isempty(visErrors)
        plot(visErrors(:,1), visErrors(:,2), 'ok', 'MarkerSize', 9);
        plot(visErrors(:,1), visErrors(:,2), 'ow', 'MarkerSize', 10.5);
        plot(visErrors(:,1), visErrors(:,2), 'ok', 'MarkerSize', 12);
        
        xlabel(['white circles around data points imply that on these ' ...
            'positions the classifier visualization is incorrect'], ...
            'FontSize', descFs, 'FontWeight', 'normal');
    end

    % set the labeling as [C1, C2, ..., Cn] for the n classes
    legendStr = cell(nClasses,1);
    for i=1:nClasses
        legendStr{i} = ['C ' int2str(i)];
    end
    %legendStr = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13'};

    
    % comment for no legend! ----------------------------
    h_legend = legend(plotHandles, legendStr); 
    %h_legend = legend(plotHandlesProtos, legendStr); 
    set(h_legend,'FontSize',fontSizeFigs); 
    
    if fig == 1
        set(h_legend,'Location','best'); drawnow;
        pos = get(h_legend, 'Position');
    else
        set(h_legend, 'Position', pos);
    end
    % --------------------------------------------------

    % pos = [0.8818 0.0319 0.0840 0.5177];
    %pos = [0.0316 0.0788 0.0970 0.6708];
    %set(h_legend, 'Position', pos);
end


%% make a plot combining class and border information
drawnow;

imRes = combinePlots(hData, hClass);
hRes = myFigure; 
imshow(imRes, [], 'Border', 'tight');
drawnow;
%pause(0.2); % pause in order to wait until the image is drawn

if ~plotIntermResult
    close(hClass);
    close(hData);
end
