function [labels, certainties] = predictCertaintyKnn(model, highDimData)
% Calculates the prediction of a k-nn model, provided by the Statistic 
% Toolbox in Matlab.
%
% The inputs:
%   model                - a model trained of the classifier
%   highDimData          - the input data to be classified (rows of the
%                          matrix)
%
% The outputs:
%   label                - the predicted labels for the points in 
%                          highDimData (a column vector)
%   certainties          - values indicating the certainty of the
%                          classification (for the predicted class); should
%                          be a column vector
%
% NOTE: This method is written for the k-nn implementation provided
% by the Statistics Toolbox in Matlab.

%
% Copyright (c) 2015 Alexander Schulz
%
% This file is part of the Classifier Visualization Toolbox.
%
% The Classifier Visualization Toolbox is distributed under the
% GNU General Public License (version 3 or later). For details, see
% <http://www.gnu.org/licenses/>.
%



[labels, certainties] = predict(model, highDimData);

certainties = addressElements(certainties, labels);

